import 'package:flutter/material.dart';

///Widget extensions to entend widget functionality
extension WidgetExtensions on Widget {
  ///Directly [Center] any widget using dot operator
  Center get center => Center(child: this);

  ///Creates a widget that expands a child of a [Row], [Column], or [Flex]
  Expanded expanded([int? flex]) => Expanded(child: this, flex: flex ?? 1);

  ///Creates a widget that controls where a child of a [Stack] is positioned.
  Positioned positioned({
    double? top,
    double? bottom,
    double? left,
    double? right,
    double? height,
    double? width,
  }) =>
      Positioned(
        child: this,
        top: top,
        bottom: bottom,
        left: left,
        right: right,
        height: height,
        width: width,
      );

  /// Hide widget
  Widget get hide => _visible(false);

  /// Show widget
  Widget get show => _visible(true);

  /// Hide widget if condition is true
  Widget hideIf(bool value) => value ? _visible(false) : _visible(true);

  /// Show widget if condition is false
  Widget showIf(bool value) => value ? _visible(true) : _visible(false);

  ///[Align] child widget
  Align align([Alignment alignment = Alignment.center]) =>
      _align(alignment: alignment);

  ///[Align] child to Bottom Right of parent widget
  Align get bottomRight => _align(alignment: Alignment.bottomRight);

  ///[Align] child to Bottom Left of parent widget
  Align get bottomLeft => _align(alignment: Alignment.bottomLeft);

  ///[Align] child to Bottom Center of parent widget
  Align get bottomCenter => _align(alignment: Alignment.bottomCenter);

  ///[Align] child to Top Right of parent widget
  Align get topRight => _align(alignment: Alignment.topRight);

  ///[Align] child to Top Left of parent widget
  Align get topLeft => _align(alignment: Alignment.topLeft);

  ///[Align] child to Top Center of parent widget
  Align get topCenter => _align(alignment: Alignment.topCenter);

  ///[Align] child to Center Right of parent widget
  Align get centerRight => _align(alignment: Alignment.centerRight);

  ///[Align] child to Center Left of parent widget
  Align get centerLeft => _align(alignment: Alignment.centerLeft);

  ///Creates a fixed size box.
  SizedBox sizedBox({double? h, double? w}) =>
      SizedBox(child: this, height: h, width: w);

  ///Create a widget that makes its child partially transparent
  Opacity opacity(double opacity) => _opacity(opacity: opacity);

  ///Get 0% [Opacity] which means fully invisible
  Opacity get opacity0 => _opacity(opacity: 0.0);

  ///Get 5% [Opacity] which means slightly visible
  Opacity get opacity5 => _opacity(opacity: 0.5);

  ///Get 10% [Opacity] which means slightly visible
  Opacity get opacity10 => _opacity(opacity: 0.10);

  ///Get 20% [Opacity] which means slightly visible
  Opacity get opacity20 => _opacity(opacity: 0.20);

  ///Get 25% [Opacity] which means paritally visible
  Opacity get opacity25 => _opacity(opacity: 0.25);

  ///Get 50% [Opacity] which means half visible
  Opacity get opacity50 => _opacity(opacity: 0.50);

  ///Get 75% [Opacity] which means paritally invisible
  Opacity get opacity75 => _opacity(opacity: 0.75);

  ///Get 100% [Opacity] which means fully visible
  Opacity get opacity100 => _opacity(opacity: 1.0);

  /// Add on Tap handler to the current widget
  GestureDetector onTap(VoidCallback callback) =>
      GestureDetector(child: this, onTap: callback);

  /// Add on Double Tap handler to the current widget
  GestureDetector onDoubleTap(VoidCallback callback) =>
      GestureDetector(child: this, onDoubleTap: callback);

  /// Add on Long Press handler to the current widget
  GestureDetector onLongPress(VoidCallback callback) =>
      GestureDetector(child: this, onLongPress: callback);

  /// Add on InkWell Tap handler to the current widget
  InkWell onInkTap(
    VoidCallback callback, {
    Color? splashColor,
    Color? focusColor,
    Color? hoverColor,
    bool? enableFeedback,
    double? radius,
    BorderRadius? borderRadius,
    MouseCursor? mouseCursor,
  }) {
    return InkWell(
      child: this,
      onTap: callback,
      splashColor: splashColor,
      focusColor: focusColor,
      hoverColor: hoverColor,
      enableFeedback: enableFeedback,
      radius: radius,
      borderRadius: borderRadius,
      mouseCursor: mouseCursor,
    );
  }

  /// Add onInkWell Double Tap handler to the current widget
  InkWell onInkDoubleTap(
    VoidCallback callback, {
    Color? splashColor,
    Color? focusColor,
    Color? hoverColor,
    bool? enableFeedback,
    double? radius,
    BorderRadius? borderRadius,
    MouseCursor? mouseCursor,
  }) {
    return InkWell(
      child: this,
      onDoubleTap: callback,
      splashColor: splashColor,
      focusColor: focusColor,
      hoverColor: hoverColor,
      enableFeedback: enableFeedback,
      radius: radius,
      borderRadius: borderRadius,
      mouseCursor: mouseCursor,
    );
  }

  Align _align({required Alignment alignment}) =>
      Align(child: this, alignment: alignment);

  Opacity _opacity({required double opacity}) =>
      Opacity(opacity: opacity, child: this);

  Visibility _visible(bool condition) =>
      Visibility(child: this, visible: condition);
}
